#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

typedef struct HidReport
{
    /*
     * +----\------+-----+-----+-----+-----+-----+-----+-----+-----+
     * |Byte \ Bit |  7  |  6  |  5  |  4  |  3  |  2  |  1  |  0  |
     * +------\----+-----+-----+-----+-----+-----+-----+-----+-----+
     * |  0        |                 Light (L)                     |
     * +-----------+-----+-----+-----+-----+-----+-----+-----+-----+
     * |  1        |            UNUSED           | Btn | Light (H) |
     * +-----------+-----+-----+-----+-----+-----+-----+-----+-----+
     *
     */
    uint16_t light:10;
    uint8_t  button1:1;
    uint8_t  button2:1;
    uint8_t  button3:1;
    uint8_t  button4:1;
    uint8_t  button5:1;
    uint8_t  unused:1;
} HidReport_t;

static HidReport_t reportBuffer;
static uchar idleRate; // Unused

/////////////////////////////////////////////////////////////////////
// USB report descriptor, generated by HID Descriptor Tool from
//    http://www.usb.org/developers/hidpage/dt2_4.zip
/////////////////////////////////////////////////////////////////////
PROGMEM char usbHidReportDescriptor[USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH] = {
    0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
    0x09, 0x04,                    // USAGE (Joystick)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x05, 0x01,                    //   USAGE_PAGE (Generic Desktop)
    0x09, 0x01,                    //   USAGE (Pointer)
    0xa1, 0x00,                    //   COLLECTION (Physical)
    0x05, 0x01,                    //     USAGE_PAGE (Generic Desktop)
    0x09, 0x30,                    //     USAGE (X)
    0x15, 0x00,                    //     LOGICAL_MINIMUM (0)
    0x26, 0xff, 0x03,              //     LOGICAL_MAXIMUM (1023)
    0x75, 0x0a,                    //     REPORT_SIZE (10)
    0x95, 0x01,                    //     REPORT_COUNT (1)
    0x81, 0x02,                    //     INPUT (Data,Var,Abs)
    0x05, 0x09,                    //     USAGE_PAGE (Button)
    0x19, 0x01,                    //     USAGE_MINIMUM (Button 1)
    0x29, 0x05,                    //     USAGE_MAXIMUM (Button 5)
    0x15, 0x00,                    //     LOGICAL_MINIMUM (0)
    0x25, 0x01,                    //     LOGICAL_MAXIMUM (1)
    0x75, 0x01,                    //     REPORT_SIZE (1)
    0x95, 0x05,                    //     REPORT_COUNT (5)
    0x81, 0x02,                    //     INPUT (Data,Var,Abs)
    0x75, 0x01,                    //     REPORT_SIZE (1)
    0x95, 0x01,                    //     REPORT_COUNT (1)
    0x81, 0x03,                    //     INPUT (Cnst,Var,Abs)
    0xc0,                          //     END_COLLECTION
    0xc0                           // END_COLLECTION
};

///////////////////////////////////////////////////////////
uint16_t read_adc(uint8_t channel)
{
    uint16_t adc_val;

    ADMUX = 0x40;
    ADMUX |= channel;
    ADCSRA = 0xC6;
    while ((ADCSRA & (1<<ADSC)))
        ;
    adc_val = ADCL + ADCH*256;
    return adc_val;
}

///////////////////////////////////////////////////////////
void buildReport()
{
    reportBuffer.button1 = ((PINC & (1<<PC0)) == 0);
    reportBuffer.button2 = ((PINC & (1<<PC1)) == 0);
    reportBuffer.button3 = ((PINC & (1<<PC2)) == 0);
    reportBuffer.button4 = ((PINC & (1<<PC4)) == 0);
    reportBuffer.button5 = ((PINC & (1<<PC5)) == 0);
    reportBuffer.light = read_adc(PC3);
}

///////////////////////////////////////////////////////////
usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;

    if(rq->bRequest == USBRQ_HID_GET_REPORT)
    {
        usbMsgPtr = (void *)&reportBuffer;
        return sizeof(reportBuffer);
    }
    else if (rq->bRequest == USBRQ_HID_GET_IDLE)
    {
        usbMsgPtr = &idleRate;
        return 1;
    }
    else if (rq->bRequest == USBRQ_HID_SET_IDLE)
    {
        idleRate = rq->wValue.bytes[1];
    }

    return 0;
}

///////////////////////////////////////////////////////////
int main(void)
{
    DDRC  = 0b000000; // all pin input
    PORTC = (1 << PC0)|(1 << PC1)|(1 << PC2)|(1 << PC4)|(1 << PC5);                 // Enable pull-up at PC3

    usbInit();

    /* enforce re-enumeration, do this while interrupts are disabled! */
    usbDeviceDisconnect();
    _delay_ms(300);
    usbDeviceConnect();

    /* enable global interrupts */
    sei();

    /* main event loop */
    for(;;)
    {
        /* always have report ready */
        buildReport();

        usbPoll();

        /* let the host know without asking */
        if (usbInterruptIsReady())
        {
            usbSetInterrupt((void *)&reportBuffer, sizeof(reportBuffer));
        }
    }

    return 0;
}

/* ------------------------------------------------------------------------- */
